"""SSM client for project"""
import subprocess
import json
import boto3
from loguru import logger
from pick import pick
from pysshm.exception import PySshmException


class Client:
    """Primary SSM client"""

    def __init__(self, profile: str, region: str):
        """__init_ SSM client with correct profile and region"""
        logger.debug("Init with {}/{}", profile, region)

        session = boto3.Session(profile_name=profile, region_name=region)

        self.region = region
        self.profile = profile

        self.client = session.client("ssm")

    def get_available_instances(self) -> str:
        """Returns the list of instances with SSM agent enable in given region"""
        logger.debug("Fetching available instances")

        response = self.client.describe_instance_information()

        instances = [
            (i["ComputerName"], i["InstanceId"], i["IPAddress"], i["PlatformName"])
            for i in response["InstanceInformationList"]
            if i["PingStatus"] == "Online"
        ]
        if len(instances) == 0:
            raise PySshmException(
                f"No instance found in {self.region} region, "
                "are you sure you're in the correct one?"
            )

        title = "Select an instance to connect to:"

        choice = pick(
            instances, title, indicator=">", options_map_func=Client.get_label
        )
        logger.debug("Chosen instance: {}", choice[0][1])

        return choice[0][1]

    @staticmethod
    def get_label(option):
        """Returns a properly formatted label for selection"""
        # pylint: disable=C0209
        return "{:<30} | {:^19} | {:^15} | {:<20} |".format(*option)

    def start_ssh_tunnel(self, config) -> None:
        """Start a SSM session and pass it to session-plugin for real
        connection
        """
        logger.debug("Starting connection to {}", config["Target"])

        session = self.client.start_session(**config)

        del session["ResponseMetadata"]

        cmd = [
            "session-manager-plugin",
            json.dumps(session),
            self.region,
            "StartSession",
            self.profile,
            json.dumps(config),
        ]

        logger.info(session)
        subprocess.run(cmd, check=True)



"""Primary entrypoint for project"""
import click
from loguru import logger
from .ssm import Client
from . import log, exception


@click.command()
@click.option(
    "-p", "--profile", default="default", envvar="AWS_PROFILE", help="AWS profile"
)
@click.option(
    "-r",
    "--region",
    default="eu-west-3",
    envvar="AWS_REGION",
    help="AWS region (default: eu-west-3)",
)
@click.option(
    "-i", "--instance-id", default=None, help="Instance ID for direct connect"
)
@click.option("-d", "--debug", is_flag=True, default=False, help="Enable debug")
def run(profile: str, region: str, instance_id: str, debug: bool) -> None:
    """Connect to an EC2 instance over SSM, all in your favorite shell."""

    log.configure(debug=debug)

    try:
        ssm_client = Client(profile=profile, region=region)

        if instance_id is None:
            # No instance provided, offer the list
            instance_id = ssm_client.get_available_instances()

        # Port forward, someday
        tunnel_config = {"Target": instance_id}

        # Init the real connection
        ssm_client.start_ssh_tunnel(tunnel_config)
    except exception.PySshmException as pse:
        logger.error(pse)
    except Exception as e:  # pylint: disable=W0703,C0103
        logger.error("An unknown error occured: {}", e)



"""Loguru configuration for project"""
import sys
from loguru import logger


def configure(debug: bool) -> None:
    """Configure loguru with correct debug level"""
    config = {
        "handlers": [
            {
                "sink": sys.stdout,
                "format": "{message}",
                "level": "DEBUG" if debug else "INFO",
            },
        ]
    }
    logger.configure(**config)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pysshm']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.18.58,<2.0.0',
 'click>=8.0.3,<9.0.0',
 'loguru>=0.5.3,<0.6.0',
 'pick>=1.0.0,<2.0.0']

entry_points = \
{'console_scripts': ['sshm = pysshm.cli:run']}

setup_kwargs = {
    'name': 'pysshm',
    'version': '0.3.0',
    'description': 'SSM shell client, the python way',
    'long_description': None,
    'author': 'Pierre-Yves Gillier',
    'author_email': 'github@pygillier.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
